@@ -87,13 +87,16 @@ import papyon.service.SingleSignOn as SSO
 import papyon.service.AddressBook as AB
 import papyon.service.OfflineIM as OIM
 import papyon.service.Spaces as Spaces
+import papyon.service.ContentRoaming as CR
 
 from papyon.util.decorator import rw_property
 from papyon.transport import *
 from papyon.switchboard_manager import SwitchboardManager
+from papyon.media import RTCActivityManager
 from papyon.msnp2p import P2PSessionManager
+from papyon.msnp2p.transport.switchboard import SwitchboardP2PTransport
 from papyon.p2p import MSNObjectStore, FileTransferManager, WebcamHandler
-from papyon.sip import SIPConnectionManager
+from papyon.sip import SIPCallManager
 from papyon.conversation import SwitchboardConversation, \
     ExternalNetworkConversation
 from papyon.event import ClientState, ClientErrorType, \
@@ -121,6 +124,7 @@ class Client(EventsDispatcher):
 
             @param proxies: proxies that we can use to connect
             @type proxies: {type: string => L{gnet.proxy.ProxyInfos}}
+            @note the key should be 'http', 'https' or 'direct'
 
             @param transport_class: the transport class to use for the network
                     connection
@@ -145,7 +149,8 @@ class Client(EventsDispatcher):
         self._webcam_handler = WebcamHandler(self)
         self._p2p_session_manager.register_handler(self._webcam_handler)
 
-        self._call_manager = SIPConnectionManager(self, self._protocol)
+        self._call_manager = SIPCallManager(self)
+        self._rtc_activity_manager = RTCActivityManager(self, self._protocol)
 
         self._msn_object_store = MSNObjectStore(self)
         self._p2p_session_manager.register_handler(self._msn_object_store)
@@ -160,6 +165,7 @@ class Client(EventsDispatcher):
         self._address_book = None
         self._oim_box = None
         self._mailbox = None
+        self._roaming = None
 
         self.__die = False
         self.__connect_transport_signals()
@@ -194,16 +200,26 @@ class Client(EventsDispatcher):
 
     @property
     def call_manager(self):
-        """The SIP connection manager
-            @type: L{SIPConnectionManager<papyon.sip.SIPConnectionManager>}"""
+        """The SIP call manager
+            @type: L{SIPCallManager<papyon.sip.SIPCallManager>}"""
         return self._call_manager
 
+    @property
+    def content_roaming(self):
+        return self._roaming
+
     @property
     def ft_manager(self):
         """The files transfer manager
             @type: L{FileTransferManager<papyon.p2p.FileTransferManager>}"""
         return self._ft_manager
 
+    @property
+    def rtc_activity_manager(self):
+        """The RTC activity manager
+            @type: L{RTCActivityManager<papyon.media.RTCActivityManager>}"""
+        return self._rtc_activity_manager
+
     @property
     def oim_box(self):
         """The offline IM for the current user
@@ -305,6 +321,12 @@ class Client(EventsDispatcher):
     ### private:
     def __connect_profile_signals(self):
         """Connect profile signals"""
+        def event(contact, *args):
+            event_name = args[-1]
+            event_args = args[:-1]
+            method_name = "on_profile_%s" % event_name.replace("-", "_")
+            self._dispatch(method_name, *event_args)
+
         def property_changed(profile, pspec):
             method_name = "on_profile_%s_changed" % pspec.name.replace("-", "_")
             self._dispatch(method_name)
@@ -314,6 +336,12 @@ class Client(EventsDispatcher):
         self.profile.connect("notify::personal-message", property_changed)
         self.profile.connect("notify::current-media", property_changed)
         self.profile.connect("notify::msn-object", property_changed)
+        self.profile.connect("notify::end-points", property_changed)
+
+        def connect_signal(name):
+            self.profile.connect(name, event, name)
+        connect_signal("end-point-added")
+        connect_signal("end-point-removed")
 
     def __connect_mailbox_signals(self):
         """Connect mailbox signals"""
@@ -346,6 +374,7 @@ class Client(EventsDispatcher):
         contact.connect("notify::current-media", property_changed)
         contact.connect("notify::msn-object", property_changed)
         contact.connect("notify::client-capabilities", property_changed)
+        contact.connect("notify::end-points", property_changed)
 
         def connect_signal(name):
             contact.connect(name, event, name)
@@ -362,6 +391,7 @@ class Client(EventsDispatcher):
             self._oim_box = OIM.OfflineMessagesBox(self._sso, self, self._proxies)
             self.__connect_oim_box_signals()
             self._spaces = Spaces.Spaces(self._sso, self._proxies)
+            self._roaming = CR.ContentRoaming(self._sso, self._address_book, self._proxies)
 
             self._state = ClientState.CONNECTED
 
@@ -438,6 +468,8 @@ class Client(EventsDispatcher):
             if handler_class is SwitchboardConversation:
                 if self._dispatch("on_invite_conversation", handler) == 0:
                     logger.warning("No event handler attached for conversations")
+            elif handler_class in [SwitchboardP2PTransport]:
+                pass
             else:
                 logger.warning("Unknown Switchboard Handler class %s" % handler_class)
 
@@ -465,6 +497,7 @@ class Client(EventsDispatcher):
             self.address_book.connect(name, event, name)
 
         connect_signal("contact-added")
+        connect_signal("contact-pending")
         connect_signal("contact-deleted")
         connect_signal("contact-blocked")
         connect_signal("contact-unblocked")
@@ -506,7 +539,7 @@ class Client(EventsDispatcher):
         def invite_received(call_manager, call):
             self._dispatch("on_invite_conference", call)
 
-        self._call_manager.connect("invite-received", invite_received)
+        self._call_manager.connect("incoming-call", invite_received)
 
     def __connect_ft_manager_signals(self):
         """Connect File Transfer Manager signals"""
