@@ -32,6 +32,7 @@ import papyon
 from papyon.gnet.message.HTTP import HTTPMessage
 from papyon.util.queue import PriorityQueue, LastElementQueue
 from papyon.util.decorator import throttled
+from papyon.util.encoding import decode_rfc2047_string
 import papyon.util.element_tree as ElementTree
 import papyon.profile as profile
 import papyon.service.SingleSignOn as SSO
@@ -74,7 +75,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
 
             "buddy-notification-received" : (gobject.SIGNAL_RUN_FIRST,
                 gobject.TYPE_NONE,
-                (object, object,)),
+                (object, object, object, object)),
 
             "mail-received" : (gobject.SIGNAL_RUN_FIRST,
                 gobject.TYPE_NONE,
@@ -135,7 +136,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         raise AttributeError, "unknown property %s" % pspec.name
 
     # Public API -------------------------------------------------------------
-    @throttled(2000, LastElementQueue())
+    @throttled(2, LastElementQueue())
     def set_presence(self, presence, client_id=0, msn_object=None):
         """Publish the new user presence.
 
@@ -152,7 +153,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
             self._send_command('CHG',
                     (presence, str(client_id), urllib.quote(str(msn_object))))
 
-    @throttled(2000, LastElementQueue())
+    @throttled(2, LastElementQueue())
     def set_display_name(self, display_name):
         """Sets the new display name
 
@@ -161,7 +162,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         self._send_command('PRP',
                 ('MFN', urllib.quote(display_name)))
 
-    @throttled(2000, LastElementQueue())
+    @throttled(2, LastElementQueue())
     def set_personal_message(self, personal_message='', current_media=None,
             signature_sound=None):
         """Sets the new personal message
@@ -191,7 +192,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
             self._client.profile._server_property_changed("current-media",
                 current_media)
 
-    @throttled(2000, LastElementQueue())
+    @throttled(2, LastElementQueue())
     def set_end_point_name(self, name="Papyon", idle=False):
         ep = '<EndpointData>'\
                 '<Capabilities>%s</Capabilities>'\
@@ -208,12 +209,16 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         self._send_command('UUX', payload=ep)
         self._send_command('UUX', payload=pep)
 
+    @throttled(2, LastElementQueue())
+    def set_privacy(self, privacy=profile.Privacy.BLOCK):
+        self._send_command("BLP", (privacy,))
+
     def signoff(self):
         """Logout from the server"""
         self._send_command('OUT')
         self._transport.lose_connection()
 
-    @throttled(7600, list())
+    @throttled(7, list())
     def request_switchboard(self, priority, callback, *callback_args):
         self.__switchboard_callbacks.add((callback, callback_args), priority)
         self._send_command('XFR', ('SB',))
@@ -270,8 +275,13 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                     (domain, user, membership, network_id)
             self._send_command("RML", payload=payload)
 
-    def send_user_notification(self, message, contact, type):
-        self._send_command("UUN", (contact.account, type), message)
+    def send_user_notification(self, message, contact, contact_guid, type,
+            callback=None, *cb_args):
+        account = contact
+        if contact_guid:
+            account += ';{' + contact_guid + '}'
+        arguments = (account, type)
+        self._send_command("UUN", arguments, message, True, callback, *cb_args)
 
     def send_unmanaged_message(self, contact, message):
         content_type = message.content_type[0]
@@ -289,7 +299,22 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         tr_id = self._send_command('URL', url_command_args)
         self._url_callbacks[tr_id] = callback
 
-    def _parse_account(self, command, idx=0):
+    # Helpers ----------------------------------------------------------------
+    def __search_account(self, account, network_id=profile.NetworkID.MSN):
+        if account == self._client.profile.account:
+            return [self._client.profile]
+
+        contacts = self._client.address_book.contacts.\
+                search_by_network_id(network_id).\
+                search_by_account(account)
+
+        if len(contacts) == 0:
+            logger.warning("Contact (network_id=%d) %s not found" % \
+                    (network_id, account))
+
+        return contacts
+
+    def __parse_network_and_account(self, command, idx=0):
         if self._protocol_version >= 18:
             temp = command.arguments[idx].split(":")
             network_id = int(temp[0])
@@ -301,6 +326,20 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         idx += 1
         return idx, network_id, account
 
+    def __parse_account_and_guid(self, command, idx=0):
+        account = command.arguments[idx]
+        guid = None
+        if ';' in account:
+            account, guid = account.split(';', 1)
+            guid = guid [1:-1]
+        return idx + 1, account, guid
+
+    def __find_node(self, parent, name, default):
+        node = parent.find(name)
+        if node is not None and node.text is not None:
+            return node.text.encode("utf-8")
+        else:
+            return default
 
     # Handlers ---------------------------------------------------------------
     # --------- Connection ---------------------------------------------------
@@ -410,35 +449,19 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         else:
             self._client.profile._server_property_changed("msn_object", None)
 
-    def _handle_ILN(self,command):
+    def _handle_ILN(self, command):
         self._handle_NLN(command)
 
-    def _handle_FLN(self,command):
-        idx, network_id, account = self._parse_account(command)
-
-        contacts = self._client.address_book.contacts.\
-                search_by_network_id(network_id).\
-                search_by_account(account)
-
-        if len(contacts) == 0:
-            logger.warning("Contact (network_id=%d) %s not found" % \
-                    (network_id, account))
-
+    def _handle_FLN(self, command):
+        idx, network_id, account = self.__parse_network_and_account(command)
+        contacts = self.__search_account(account, network_id)
         for contact in contacts:
+            contact._remove_flag(profile.ContactFlag.EXTENDED_PRESENCE_KNOWN)
             contact._server_property_changed("presence",
                     profile.Presence.OFFLINE)
 
-    def _handle_NLN(self,command):
-        idx, network_id, account = self._parse_account(command, 1)
-
-        contacts = self._client.address_book.contacts.\
-                search_by_network_id(network_id).\
-                search_by_account(account)
-
-        if len(contacts) == 0:
-            logger.warning("Contact (network_id=%d) %s not found" % \
-                    (network_id, account))
-
+    def _handle_NLN(self, command):
+        idx, network_id, account = self.__parse_network_and_account(command, 1)
         presence = command.arguments[0]
         display_name = urllib.unquote(command.arguments[idx])
         idx += 1
@@ -447,19 +470,27 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
 
         msn_object = None
         icon_url = None
+
         if len(command.arguments) > idx:
             if command.arguments[idx] not in ('0', '1'):
                 msn_object = papyon.p2p.MSNObject.parse(self._client,
                                urllib.unquote(command.arguments[idx]))
+
         idx += 1
         if len(command.arguments) > idx:
             icon_url = command.arguments[idx]
 
+        contacts = self.__search_account(account, network_id)
         for contact in contacts:
-            contact._server_property_changed("presence", presence)
+            # don't change local presence and capabilities
+            if contact is not self._client.profile:
+                contact._server_property_changed("presence", presence)
+                contact._server_property_changed("client-capabilities", capabilities)
             contact._server_property_changed("display-name", display_name)
-            contact._server_property_changed("client-capabilities", capabilities)
-            contact._server_property_changed("msn-object", msn_object)
+            # only change MSNObject if the extended presence is known (MSNP18)
+            if self._protocol_version < 18 or \
+               contact.has_flag(profile.ContactFlag.EXTENDED_PRESENCE_KNOWN):
+                contact._server_property_changed("msn_object", msn_object)
             if icon_url is not None:
                 contact._server_attribute_changed('icon_url', icon_url)
 
@@ -475,17 +506,20 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
     def _handle_UUX(self, command):
         pass
 
-    def _handle_UBN(self,command): # contact infos
+    def _handle_UBN(self, command): # contact infos
         if not command.payload:
             return
+        idx, account, guid = self.__parse_account_and_guid(command)
+        contact = self.__search_account(account)[0]
         type = int(command.arguments[1])
-        self.emit("buddy-notification-received", type, command)
+        payload = command.payload
+        self.emit("buddy-notification-received", contact, guid, type, payload)
 
-    def _handle_UBX(self,command): # contact infos
+    def _handle_UBX(self, command): # contact infos
         if not command.payload:
             return
-
-        idx, network_id, account = self._parse_account(command)
+        print command.payload
+        idx, network_id, account = self.__parse_network_and_account(command)
 
         try:
             tree = ElementTree.fromstring(command.payload)
@@ -493,42 +527,48 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
             logger.error("Invalid XML data in received UBX command")
             return
 
-        cm = tree.find("./CurrentMedia")
-        if cm is not None and cm.text is not None:
-            parts = cm.text.split('\\0')
-            if parts[1] == 'Music' and parts[2] == '1':
-                cm = (parts[4].encode("utf-8"), parts[5].encode("utf-8"))
-            elif parts[2] == '0':
-                cm = None
-        else:
+        utl = self.__find_node(tree, "./UserTileLocation", "")
+        cm_parts = self.__find_node(tree, "./CurrentMedia", "").split('\\0')
+        pm = self.__find_node(tree, "./PSM", "")
+        rmu = self.__find_node(tree, "./RMU", "")
+        ss = self.__find_node(tree, "./SignatureSound", None)
+        mg = self.__find_node(tree, "./MachineGuid", "{}").lower()[1:-1]
+
+        msn_object = None
+        if utl != "":
+            msn_object = papyon.p2p.MSNObject.parse(self._client, utl)
+
+        if len(cm_parts) < 3 or cm_parts[2] == '0':
             cm = None
+        elif cm_parts[1] == 'Music' and cm_parts[2] == '1':
+            cm = (cm_parts[4], cm_parts[5])
 
-        pm = tree.find("./PSM")
-        if pm is not None and pm.text is not None:
-            pm = pm.text.encode("utf-8")
-        else:
-            pm = ""
-
-        ss = tree.find("./SignatureSound")
-        if ss is not None and ss.text is not None:
-            ss = ss.text.encode("utf-8")
-        else:
-            ss = None
-
-        contacts = self._client.address_book.contacts.\
-                search_by_network_id(network_id).\
-                search_by_account(account)
-
-        if len(contacts) == 0:
-            logger.warning("Contact (network_id=%d) %s not found" % \
-                    (network_id, account))
+        eps = tree.findall("./EndpointData")
+        end_points = {}
+        for ep in eps:
+            guid = ep.get("id").encode("utf-8")[1:-1]
+            caps = self.__find_node(ep, "Capabilities", "0:0")
+            end_points[guid] = profile.EndPoint(guid, caps)
+        peps = tree.findall("./PrivateEndpointData")
+        for pep in peps:
+            guid = pep.get("id").encode("utf-8")[1:-1]
+            if guid not in end_points: continue
+            end_point = end_points[guid]
+            end_point.name = self.__find_node(pep, "EpName", "")
+            end_point.idle = bool(self.__find_node(pep, "Idle", "").lower() == "true")
+            end_point.client_type = int(self.__find_node(pep, "ClientType", 0))
+            end_point.state = self.__find_node(pep, "State", "")
 
+        contacts = self.__search_account(account, network_id)
         for contact in contacts:
+            contact._add_flag(profile.ContactFlag.EXTENDED_PRESENCE_KNOWN)
+            contact._server_property_changed("msn-object", msn_object)
             contact._server_property_changed("current-media", cm)
             contact._server_property_changed("personal-message", pm)
             contact._server_property_changed("signature-sound", ss)
+            contact._server_property_changed("end-points", end_points)
 
-    def _handle_UUN(self,command): # UBN acknowledgment
+    def _handle_UUN(self, command): # UBN acknowledgment
         pass
 
     # --------- Contact List -------------------------------------------------
@@ -559,8 +599,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                     profile[name] = value.strip()
             self._client.profile._server_property_changed("profile", profile)
 
-            self._send_command("BLP",
-                    (self._client.profile.privacy,))
+            self.set_privacy(self._client.profile.privacy)
             self._state = ProtocolState.SYNCHRONIZING
             self._client.address_book.sync()
         elif content_type[0] in \
@@ -590,9 +629,9 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
             #New mail
             m = HTTPMessage()
             m.parse(message.body)
-            name = m.get_header('From')
+            name = decode_rfc2047_string(m.get_header('From'))
             address = m.get_header('From-Addr')
-            subject = m.get_header('Subject')
+            subject = decode_rfc2047_string(m.get_header('Subject'))
             message_url = m.get_header('Message-URL')
             post_url = m.get_header('Post-URL')
             post_id = m.get_header('id')
@@ -616,16 +655,9 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                 self._client.mailbox._unread_mail_increased(delta)
 
     def _handle_UBM(self, command):
-        idx, network_id, account = self._parse_account(command)
-
-        contacts = self._client.address_book.contacts.\
-                search_by_network_id(network_id).\
-                search_by_account(account)
-
-        if len(contacts) == 0:
-            logger.warning("Contact (network_id=%d) %s not found" % \
-                    (network_id, account))
-        else:
+        idx, network_id, account = self.__parse_network_and_account(command)
+        contacts = self.__search_account(account, network_id)
+        if len(contacts) > 0:
             contact = contacts[0]
             message = Message(contact, command.payload)
             self.emit("unmanaged-message-received", contact, message)
@@ -671,7 +703,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
             callback(post_url, form_dict)
 
     # --------- Invitation ---------------------------------------------------
-    def _handle_RNG(self,command):
+    def _handle_RNG(self, command):
         session_id = command.arguments[0]
         host, port = command.arguments[1].split(':',1)
         port = int(port)
@@ -684,17 +716,21 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         self.emit("switchboard-invitation-received", session, inviter)
 
     # --------- Challenge ----------------------------------------------------
-    def _handle_QNG(self,command):
+    def _handle_QNG(self, command):
         pass
 
-    def _handle_QRY(self,command):
+    def _handle_QRY(self, command):
         pass
 
-    def _handle_CHL(self,command):
+    def _handle_CHL(self, command):
         response = _msn_challenge(command.arguments[0])
         self._send_command('QRY',
                 (ProtocolConstant.PRODUCT_ID,), payload=response)
 
+    # --------- Notification -------------------------------------------------
+    def _handle_NOT(self, command):
+        pass
+
     # callbacks --------------------------------------------------------------
     def _connect_cb(self, transport):
         self.__switchboard_callbacks = PriorityQueue()
@@ -733,16 +769,9 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                 address_book.profile.display_name)
 
         contacts = address_book.contacts.group_by_domain()
-        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
-
-        for contact in address_book.contacts:
-            if (contact.memberships & mask & ~profile.Membership.FORWARD) == \
-                    (profile.Membership.ALLOW | profile.Membership.BLOCK):
-                logger.warning("Contact is on both Allow and Block list; " \
-                               "removing from Allow list (%s)" % contact.account)
-                contact._remove_membership(profile.Membership.ALLOW)
 
         payloads = ['<ml l="1">']
+        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
         for domain, contacts in contacts.iteritems():
             payloads[-1] += '<d n="%s">' % domain
             for contact in contacts:
