@@ -21,7 +21,7 @@
 
 from papyon.gnet.message.HTTP import HTTPMessage
 from papyon.msnp2p.exceptions import ParseError
-from papyon.msnp2p.constants import SLPContentType
+from papyon.msnp2p.constants import SLPContentType, SLPStatus
 
 import base64
 import uuid
@@ -35,7 +35,8 @@ __all__ = ['SLPMessage', 'SLPRequestMessage', 'SLPResponseMessage',
 class SLPMessage(HTTPMessage):
     STD_HEADERS = ["To", "From", "Via", "CSeq", "Call-ID", "Max-Forwards"]
 
-    def __init__(self, to="", frm="", branch="", cseq=0, call_id="", max_forwards=0):
+    def __init__(self, to="", frm="", branch="", cseq=0, call_id="",
+            max_forwards=0, on_behalf=""):
         HTTPMessage.__init__(self)
         self.add_header("To", "<msnmsgr:%s>" % to)
         self.add_header("From", "<msnmsgr:%s>" % frm)
@@ -45,6 +46,8 @@ class SLPMessage(HTTPMessage):
         if call_id:
             self.add_header("Call-ID", call_id)
         self.add_header("Max-Forwards", str(max_forwards))
+        if on_behalf:
+            self.add_header("On-Behalf", on_behalf)
 
         # Make the body a SLP Message wih "null" content type
         self.body = SLPNullBody()
@@ -374,13 +377,18 @@ class SLPTransferResponseBody(SLPMessageBody):
 SLPMessageBody.register_content(SLPContentType.TRANSFER_RESPONSE, SLPTransferResponseBody)
 
 class SLPSessionCloseBody(SLPMessageBody):
-    def __init__(self, context=None, session_id=None, s_channel_state=None,
-            capabilities_flags=None):
+    def __init__(self, context=None, session_id=None, reason=None,
+            s_channel_state=None, capabilities_flags=None):
         SLPMessageBody.__init__(self, SLPContentType.SESSION_CLOSE,
                 session_id, s_channel_state, capabilities_flags)
 
         if context is not None:
             self.add_header("Context",  base64.b64encode(context));
+        if reason is not None:
+            if reason[0] == SLPStatus.ACCEPTED:
+                self.add_header("AcceptedBy", "{%s}" % reason[1].upper())
+            elif reason[0] == SLPStatus.DECLINED:
+                self.add_header("DeclinedBy", "{%s}" % reason[1].upper())
 
     @property
     def context(self):
