@@ -28,6 +28,7 @@ import gobject
 import weakref
 
 import papyon.msnp as msnp
+from papyon.profile import Presence
 from papyon.transport import ServerType
 from papyon.util.weak import WeakSet
 from papyon.event import ConversationErrorType, ContactInviteError, MessageError
@@ -36,23 +37,25 @@ __all__ = ['SwitchboardManager']
 
 logger = logging.getLogger('papyon.protocol.switchboard_manager')
 
-class SwitchboardClient(object):
-    def __init__(self, client, contacts, priority=99):
+class SwitchboardHandler(object):
+    def __init__(self, client, switchboard, contacts, priority=99):
         self._client = client
         self._switchboard_manager = weakref.proxy(self._client._switchboard_manager)
         self.__switchboard = None
         self._switchboard_requested = False
         self._switchboard_priority = priority
 
-        self._pending_invites = set(contacts)
-        self._pending_messages = []
-        self._delivery_callbacks = {}
-
-        if self._client.protocol_version >= 16:
-            self._pending_invites.add(self._client.profile)
-
         self.participants = set()
-        self._process_pending_queues()
+        self._pending_invites = set()
+        self._pending_messages = []
+        self._pending_handles = {}
+        self._delivery_callbacks = {} # transaction_id => (callback, errback)
+
+        for contact in contacts:
+            self.__add_pending(contact)
+
+        if switchboard is not None:
+            self._switchboard = switchboard
 
     @staticmethod
     def _can_handle_message(message, switchboard_client=None):
@@ -72,6 +75,8 @@ class SwitchboardClient(object):
 
         self.switchboard.connect("notify::inviting",
                 lambda sb, pspec: self.__on_user_inviting_changed())
+        self.switchboard.connect("notify::state",
+                lambda sb, pspec: self.__on_switchboard_state_changed())
         self.switchboard.connect("user-joined",
                 lambda sb, contact: self.__on_user_joined(contact))
         self.switchboard.connect("user-left",
@@ -82,7 +87,8 @@ class SwitchboardClient(object):
                 lambda sb, trid: self.__on_message_delivered(trid))
         self.switchboard.connect("message-undelivered",
                 lambda sb, trid: self.__on_message_undelivered(trid))
-        logger.info("New switchboard attached")
+        logger.info("Handler %s attached to switchboard %s" %
+                (repr(self), switchboard.session_id))
         def process_pending_queues():
             self._process_pending_queues()
             return False
@@ -93,7 +99,7 @@ class SwitchboardClient(object):
 
     # protected
     def _send_message(self, content_type, body, headers={},
-            ack=msnp.MessageAcknowledgement.HALF, callback=None, cb_args=()):
+            ack=msnp.MessageAcknowledgement.HALF, callback=None, errback=None):
         message = msnp.Message(self._client.profile)
         message.add_header('MIME-Version', '1.0')
         message.content_type = content_type
@@ -101,11 +107,11 @@ class SwitchboardClient(object):
             message.add_header(key, value)
         message.body = body
 
-        self._pending_messages.append((message, ack, callback, cb_args))
+        self._pending_messages.append((message, ack, callback, errback))
         self._process_pending_queues()
 
     def _invite_user(self, contact):
-        self._pending_invites.add(contact)
+        self.__add_pending(contact)
         self._process_pending_queues()
 
     def _leave(self):
@@ -124,41 +130,79 @@ class SwitchboardClient(object):
     def _on_contact_left(self, contact):
         raise NotImplementedError
 
+    def _on_switchboard_closed(self):
+        raise NotImplementedError
+
+    def _on_closed(self):
+        raise NotImplementedError
+
     def _on_error(self, error_type, error):
         raise NotImplementedError
 
     # private
+    def __add_pending(self, contact):
+        if contact in self._pending_invites or contact is self._client.profile:
+            return
+        self._pending_invites.add(contact)
+        handle = contact.connect("notify::presence",
+                lambda contact, pspec: self.__on_user_presence_changed(contact))
+        self._pending_handles[contact] = handle
+
+    def __remove_pending(self, contact):
+        self._pending_invites.discard(contact)
+        if contact in self._pending_handles:
+            contact.disconnect(self._pending_handles[contact])
+            del self._pending_handles[contact]
+
     def __on_user_inviting_changed(self):
         if not self.switchboard.inviting:
             self._process_pending_queues()
 
     def __on_user_joined(self, contact):
         self.participants.add(contact)
-        self._pending_invites.discard(contact)
+        self.__remove_pending(contact)
         self._on_contact_joined(contact)
 
     def __on_user_left(self, contact):
         self._on_contact_left(contact)
         self.participants.remove(contact)
         if len(self.participants) == 0:
-            self._pending_invites.add(contact)
-            try:
-                self._switchboard.leave()
-            except:
-                pass
+            self.__add_pending(contact)
+
+    def __on_user_presence_changed(self, contact):
+        if (self._switchboard and self.switchboard.state == msnp.ProtocolState.OPEN) or self._switchboard_requested:
+            return
+        for contact in self._pending_invites:
+            if contact.presence != Presence.OFFLINE:
+                return
+        # Switchboard was already closed and there is (almost) no chance
+        # a new one will be created now.
+        logger.info("All pending invites are now offline, closing handler")
+        self._leave()
+
+    def __on_switchboard_state_changed(self):
+        if self._switchboard.state == msnp.ProtocolState.CLOSED:
+            for contact in self._pending_invites:
+                if contact.presence != Presence.OFFLINE:
+                    return
+            # Might be that an "appear offline" contact closed the
+            # switchboard or that the switchboard has been explicitely closed
+            self._leave()
 
     def __on_user_invitation_failed(self, contact):
-        self._pending_invites.discard(contact)
+        self.__remove_pending(contact)
         self._on_error(ConversationErrorType.CONTACT_INVITE,
                 ContactInviteError.NOT_AVAILABLE)
 
-    def __on_message_delivered(self, transaction_id):
-        if transaction_id in self._delivery_callbacks:
-            callback, cb_args = self._delivery_callbacks.pop(transaction_id)
-            if callback:
-                callback(*cb_args)
+    def __on_message_delivered(self, trid):
+        callback, errback = self._delivery_callbacks.pop(trid, (None, None))
+        if callback:
+            callback[0](*callback[1:])
 
-    def __on_message_undelivered(self, transaction_id):
+    def __on_message_undelivered(self, trid):
+        callback, errback = self._delivery_callbacks.pop(trid, (None, None))
+        if errback:
+            errback[0](*errback[1:])
         self._on_error(ConversationErrorType.MESSAGE,
                 MessageError.DELIVERY_FAILED)
 
@@ -174,17 +218,20 @@ class SwitchboardClient(object):
         for contact in self._pending_invites:
             if contact not in self.participants:
                 self.switchboard.invite_user(contact)
+        for contact, handle in self._pending_handles.items():
+            contact.disconnect(handle)
         self._pending_invites = set()
+        self._pending_handles = dict()
 
         if not self.switchboard.inviting:
-            for message, ack, callback, cb_args in self._pending_messages:
+            for message, ack, callback, errback in self._pending_messages:
                 # if ack type is FULL or MSNC, wait for ACK before calling back
                 if ack in (msnp.MessageAcknowledgement.FULL,
                            msnp.MessageAcknowledgement.MSNC):
                     transaction_id = self.switchboard.send_message(message, ack)
-                    self._delivery_callbacks[transaction_id] = (callback, cb_args)
+                    self._delivery_callbacks[transaction_id] = (callback, errback)
                 else:
-                    self.switchboard.send_message(message, ack, callback, cb_args)
+                    self.switchboard.send_message(message, ack, callback)
 
             self._pending_messages = []
 
@@ -194,9 +241,9 @@ class SwitchboardClient(object):
             return False
         if self._switchboard_requested:
             return True
-        logger.info("requesting new switchboard")
         self._switchboard_requested = True
-        self._pending_invites |= self.participants
+        for participant in self.participants:
+            self.__add_pending(participant)
         self.participants = set()
         self._switchboard_manager.request_switchboard(self, self._switchboard_priority) # may set the switchboard immediatly
         return self._switchboard_requested
@@ -224,6 +271,7 @@ class SwitchboardManager(gobject.GObject):
         self._orphaned_handlers = WeakSet()
         self._switchboards = {}
         self._orphaned_switchboards = set()
+        self._requested_switchboards = {}
         self._pending_switchboards = {}
 
         self._client._protocol.connect("switchboard-invitation-received",
@@ -242,6 +290,8 @@ class SwitchboardManager(gobject.GObject):
 
     def request_switchboard(self, handler, priority=99):
         handler_participants = handler.total_participants
+        participants = ", ".join(map(lambda c: c.account, handler_participants))
+        logger.info("Requesting switchboard for participant(s) %s" % participants)
 
         # If the Handler was orphan, then it is no more
         self._orphaned_handlers.discard(handler)
@@ -250,6 +300,8 @@ class SwitchboardManager(gobject.GObject):
         for switchboard in self._switchboards.keys():
             switchboard_participants = set(switchboard.participants.values())
             if handler_participants == switchboard_participants:
+                logger.info("Using already opened switchboard %s" %
+                        switchboard.session_id)
                 self._switchboards[switchboard].add(handler)
                 handler._switchboard = switchboard
                 return
@@ -258,25 +310,38 @@ class SwitchboardManager(gobject.GObject):
         for switchboard in list(self._orphaned_switchboards):
             switchboard_participants = set(switchboard.participants.values())
             if handler_participants == switchboard_participants:
+                logger.info("Using orphaned switchboard %s" %
+                        switchboard.session_id)
                 self._switchboards[switchboard] = set([handler]) #FIXME: WeakSet ?
                 self._orphaned_switchboards.discard(switchboard)
                 handler._switchboard = switchboard
                 return
 
-        # Check being requested switchboards
+        # Check pending switchboards
         for switchboard, handlers in self._pending_switchboards.iteritems():
             pending_handler = handlers.pop()
             handlers.add(pending_handler)
             switchboard_participants = pending_handler.total_participants
             if handler_participants == switchboard_participants:
                 self._pending_switchboards[switchboard].add(handler)
+                logger.info("Using pending switchboard")
                 return
 
-        self._client._protocol.\
-                request_switchboard(priority, self._ns_switchboard_request_response, handler)
+        # Check switchboards being requested for same participants
+        if participants in self._requested_switchboards:
+            self._requested_switchboards[participants].add(handler)
+            logger.info("Using already requested switchboard for same contacts")
+            return
+
+        logger.info("Requesting new switchboard")
+        self._requested_switchboards[participants] = set([handler])
+        self._client._protocol.request_switchboard(priority,
+                self._ns_switchboard_request_response, participants)
 
     def close_handler(self, handler):
+        logger.info("Closing switchboard handler %s" % repr(handler))
         self._orphaned_handlers.discard(handler)
+        handler._on_closed()
         for switchboard in self._switchboards.keys():
             handlers = self._switchboards[switchboard]
             handlers.discard(handler)
@@ -292,9 +357,10 @@ class SwitchboardManager(gobject.GObject):
                 del self._pending_switchboards[switchboard]
                 self._orphaned_switchboards.add(switchboard)
 
-    def _ns_switchboard_request_response(self, session, handler):
+    def _ns_switchboard_request_response(self, session, participants):
         switchboard = self._build_switchboard(session)
-        self._pending_switchboards[switchboard] = set([handler]) #FIXME: WeakSet ?
+        handlers = self._requested_switchboards.pop(participants, set())
+        self._pending_switchboards[switchboard] = handlers
 
     def _ns_switchboard_invite(self, protocol, session, inviter):
         switchboard = self._build_switchboard(session)
@@ -350,6 +416,7 @@ class SwitchboardManager(gobject.GObject):
             if switchboard in self._switchboards.keys():
                 for handler in self._switchboards[switchboard]:
                     self._orphaned_handlers.add(handler)
+                    handler._on_switchboard_closed()
                 del self._switchboards[switchboard]
             self._orphaned_switchboards.discard(switchboard)
 
@@ -367,9 +434,9 @@ class SwitchboardManager(gobject.GObject):
                     continue
                 if not handler_class._can_handle_message(message):
                     continue
-                handler = handler_class(self._client, (), *extra_args)
+                handler = handler_class.handle_message(self._client,
+                        switchboard, message, *extra_args)
                 handlers.add(handler)
-                handler._switchboard = switchboard
                 self.emit("handler-created", handler_class, handler)
                 handler._on_message_received(message)
 
@@ -377,9 +444,9 @@ class SwitchboardManager(gobject.GObject):
             for handler_class, extra_args in self._handlers_class:
                 if not handler_class._can_handle_message(message):
                     continue
-                handler = handler_class(self._client, (), *extra_args)
+                handler = handler_class.handle_message(self._client,
+                        switchboard, message, *extra_args)
                 self._switchboards[switchboard] = set([handler]) #FIXME: WeakSet ?
                 self._orphaned_switchboards.discard(switchboard)
-                handler._switchboard = switchboard
                 self.emit("handler-created", handler_class, handler)
                 handler._on_message_received(message)
